#include 'Basic_Declarations.h'
#include 'Mergesort.h'
#include 'Closest_Pair.h'
(* procedure ClosestPair(    A: Points;
                                  { by value, so as not to destroy the data }
                             n: integer;
                         var p1, p2: point; { two closest points }
                         var d: real { their distance apart } ); *)
procedure ClosestPair;
  (* Find the closest pair from among n points in O(n log n) time. *)
  (* p1, p2 and d are global variables. *)
  type SP = record (* subproblem description *)
              (* All points within d of the boundaries of the vertical strip
                 defined by [MinX,MaxX] are in A[start..stop], sorted by their
                 ordinates.  Because d is monotone nonincreasing, the
                 implication is only one way, i.e., some of the points in
                 A[start..stop] may no longer be within d of the boundary. *)
              start, stop: integer;
              MinX, MaxX: real
            end;
  var work: Points;
      SubProbs: array [1..MaxPoints] of SP;
      i, j, k: integer;
  procedure Merge(SP1, SP2: SP; var NewSP: SP);
    label 1;
    var i, i1, i2, j, k: integer;
    begin
      (* The dividing line is really a no-man's-land between SP1.MaxX
         and SP2.MinX. *)
      (* While respecting the ordering on ordinates, scan both lists, placing
         the points in SP1 within d of SP2.MinX and the points in SP2 within d
         of SP1.MaxX at the front of the work array. *)
      i1 := SP1.start; i2 := SP2.start; k := 0;
      while (i1 <= SP1.stop) and (i2 <= SP2.stop) do
        if A[i1].y <= A[i2].y
          then begin
                 if A[i1].x >= SP2.MinX - d
                   then begin
                          k := k + 1; work[k] := A[i1]
                        end;
                 i1 := i1 + 1
               end
          else begin
                 if A[i2].x <= SP1.MaxX + d
                   then begin
                          k := k + 1; work[k] := A[i2]
                        end;
                 i2 := i2 + 1
               end;
      while i1 <= SP1.stop do (* Copy any remaining relevant elements. *)
        begin
          if A[i1].x >= SP2.MinX - d
            then begin
                   k := k + 1; work[k] := A[i1]
                 end;
          i1 := i1 + 1
        end;
      while i2 <= SP2.stop do (* Copy any remaining relevant elements. *)
        begin
          if A[i2].x <= SP1.MaxX + d
            then begin
                   k := k + 1; work[k] := A[i2]
                 end;
          i2 := i2 + 1
        end;

      (* Scan the work array, updating global variables p1, p2 and d.
         This takes constant time per point. *)
      i := 1;
      while i < k do
        begin
          for j := i + 1 to k do
            begin
              if work[j].y - work[i].y > d then goto 1;
              if Dist(work[i],work[j]) < d
                then begin p1 := work[i]; p2 := work[j]; d := Dist(p1,p2) end
            end;
       1: i := i + 1
        end;

      (* Merge the two sorted lists, retaining only points within d of the
         boundaries of the new vertical strip. *)
      i1 := SP1.start; i2 := SP2.start; k := 0; j := n + 1;
      while (i1 <= SP1.stop) and (i2 <= SP2.stop) do
        if A[i1].y <= A[i2].y
          then begin
                 if (A[i1].x <= SP1.MinX + d) or (A[i1].x >= SP2.MaxX - d)
                   then begin
                          k := k + 1; work[k] := A[i1]
                        end;
                 i1 := i1 + 1
               end
          else begin
                 if (A[i2].x <= SP1.MinX + d) or (A[i2].x >= SP2.MaxX - d)
                   then begin
                          k := k + 1; work[k] := A[i2]
                        end;
                 i2 := i2 + 1
               end;
      while i1 <= SP1.stop do (* Copy any remaining relevant elements. *)
        begin
          if (A[i1].x <= SP1.MinX + d) or (A[i1].x >= SP2.MaxX - d)
            then begin
                   k := k + 1; work[k] := A[i1]
                 end;
          i1 := i1 + 1
        end;
      while i2 <= SP2.stop do (* Copy any remaining relevant elements. *)
        begin
          if (A[i2].x <= SP1.MinX + d) or (A[i2].x >= SP2.MaxX - d)
            then begin
                   k := k + 1; work[k] := A[i2]
                 end;
          i2 := i2 + 1
        end;
      i1 := SP1.start;
      for i := 1 to k do begin A[i1] := work[i]; i1 := i1 + 1 end;
      for i := j to n do begin A[i1] := work[i]; i1 := i1 + 1 end;
      NewSP.start := SP1.start;
      NewSP.stop := i1 - 1;
      NewSP.MinX := SP1.MinX;
      NewSP.MaxX := SP2.MaxX
    end; (* Merge *)

  begin (* ClosestPair *)
    MergeSort(A,work,n); (* Sort the points by their abscissae. *)
    (* Define the initial subproblems. *)
    d := infinity;
    for i := 1 to n do
      begin
        SubProbs[i].start := i;
        SubProbs[i].stop := i;
        SubProbs[i].MinX := A[i].x;
        SubProbs[i].MaxX := A[i].x
      end;
    i := n;
    while i > 1 do
      begin
        j := 1; k := 1;
        while j+1 <= i do
          begin
            Merge(SubProbs[j],SubProbs[j+1],SubProbs[k]);
            j := j + 2; k := k + 1
          end;
        if j = i then begin SubProbs[k] := SubProbs[j]; k := k + 1 end;
        i := k - 1
      end
  end; (* ClosestPair *)
