#include 'Basic_Declarations.h'
#include 'Mergesort.h'

(* procedure MergeSort(var A, work: Points; N: integer); *)
procedure MergeSort;
  (* A non-recursive, non-stack version of mergesort based on bottom-up
     divide-and-conquer schema.

     Procedure cannot, as a matter of principle, know how long an array
     to declare for temporary storage.  Standard solution is to have user
     pass in work array.
  *)
  var span: integer; (* current length of sorted lists *)
      inA: boolean;
      i: integer;

  procedure Merge(var inarray, (* array containing lists of length  span  *)
                      outarray: Points; (* array containing lists of length
                                           2*span  upon return *)
                      N, (* total number of elements *)
                      span: integer);
    (* Merge pairs of sorted lists of length  span.  Merge operation is
       performed slightly more efficiently than in Figure|\*(yF.
    *)
    label 99;
    var i, (* points to first unprocessed element in sublist 1 *)
        j, (* points to first unprocessed element in sublist 2 *)
        k, (* points to first free space in  outarray  *)
        t, (* location past first sublist/start of second sublist *)
        u: (* location past second sublist *)
          integer;
    begin
      (* There are at least two lists to merge. *)
      i := 1;
      j := span + 1;
      k := 1;

      repeat (* until there is one left over list or no lists left at all *)
        (* invariant: k = i at start of merge of two lists *)
        t := j; (* set upper bounds for i and j *)
        u := j + span;
        if u > N
          then u := N + 1; (* second sublist is small *)

        (* perform merge *)
    99: if inarray[i].x <= inarray[j].x
          then begin
                 outarray[k] := inarray[i];
                 i := i + 1;
                 k := k + 1;
                 if i < t (* only check cursor that changed *)
                   then goto 99
                   else repeat (* copy remainder of sublist 2 *)
                          (* k = j at this point *)
                          outarray[k] := inarray[k];
                          k := k + 1
                        until k = u
               end
          else begin
                 outarray[k] := inarray[j];
                 j := j + 1;
                 k := k + 1;
                 if j < u (* only check cursor that changed *)
                   then goto 99
                   else repeat (* copy remainder of sublist 1 *)
                          outarray[k] := inarray[i];
                          i := i + 1;
                          k := k + 1
                        until k = u
               end;
        i := u;
        j := i + span
      until j > N; (* until sublist 2 is non-existent *)

      while i <= N do (* copy over an unmatched list if one exists *)
        begin
          outarray[i] := inarray[i];
          i := i + 1
        end
    end; (* Merge *)

  begin (* Mergesort *)
    inA := true;
    span := 1; (* The array consists of N sorted sublists of length one. *)
    while span < N do (* while cannot guarantee array is sorted *)
      begin
          if inA
            then Merge(A,work,N,span)
            else Merge(work,A,N,span);
        inA := not inA;
        span := 2*span
      end;
    if not inA then for i := 1 to N do A[i] := work[i]
  end; (* Mergesort *)
