  (* Conventions:
       For ordering points -- primary key is x, secondary key is y 
         less:     < 0
         equal:    = 0
         greater:  > 0

       For determining on which side of a ray a point lies or, phrased
       algebraically, whether a point falls in the positive or negative
       half-plane determined by an oriented line.
         left or negative:  < 0
         on ray or line:    = 0
         right or positive: > 0

       For determining if two points lie on the same or opposite sides of a
       line.
         same side:          > 0
         one or both points  = 0
           lie on the line:
         opposite sides:     < 0

       For determining the relationship between a point and a simple polygon.
         inside:  > 0
         onpoly:  = 0
         outside: < 0
*)

  type point = record
                 x,y: real
               end;
       segment = record
                   e1,e2: point (* endpoints of segment -- no order implied *)
                 end;

  function Compare(p1,p2: point): real; external;

  function LesserOf(p1,p2: point): point; external;

  function GreaterOf(p1,p2: point): point; external;

  function WhichSide(p: point; s: segment): real; external;

  function Sides(p1,p2: point; s: segment): real; external;
