#include 'Basic_Declarations.h'

  (* function Compare(p1,p2: point): real; *)
  function Compare;
    (* Sort on x-coordinate, or in event of tie, y-coordinate *)
    var comp: real;
    begin
      comp := p1.x - p2.x;
      if comp = 0 (* in case of tie on x-coordinate, use y-coordinate *)
        then comp := p1.y - p2.y;
      Compare := comp
    end; (* Compare *)

  (* function LesserOf(p1,p2: point): point; *)
  function LesserOf;
    (* return the lesser of the two points *)
    begin
      if Compare(p1,p2) <= 0
        then LesserOf := p1
        else LesserOf := p2
    end; (* LesserOf *)

  (* function GreaterOf(p1,p2: point): point; *)
  function GreaterOf;
    (* return the greater of the two points *)
    begin
      if Compare(p1,p2) > 0
        then GreaterOf := p1
        else GreaterOf := p2
    end; (* GreaterOf *)

  (* function WhichSide(p: point; s: segment): real; *)
  function WhichSide;
    (* Given a ray from P1 passing through P2, determine which side of
       the the point p is on.  This is done from the point of view of
       standing on P1 and looking toward P2.  If the angle (in the
       counter-clockwise direction) from P2 to P1 (apex) to p is in (0,180)
       then the result is "left", if the angle is between (180,360) then the
       result is "right", and if either 0 or 180 the result is "on".

       If the ray degenerates to a point, then the point is always on the ray,
       i.e., we can imagine the ray as pointing in any direction.
    *)
    (* Use parametric version of line:

         (x1,y1) + t(x2-x1,y2-y1)

       to determine which side the point is on.  Effectively solve for x
       coordinate of the point and compare the y coordinates to determine
       the situation.
    *)
    var a,b: real;
    begin
      a := (p.x - s.e1.x)*(s.e2.y - s.e1.y);
      b := (p.y - s.e1.y)*(s.e2.x - s.e1.x);
      WhichSide := a - b
    end; (* WhichSide *)

  (* function Sides(p1,p2: point; s: segment): real; *)
  function Sides;
    (* Determine if two points lie on the same or opposite sides of a line. *)
    begin
      Sides := WhichSide(p1,s) * WhichSide(p2,s)
    end; (* Sides *)
