program convextest(input,output);
#include 'Basic_Declarations.h'
#include 'Convex_Hull_Decls.h'
#include 'Package_Wrapping.h'
#include 'Package_Wrapping2.h'
#include 'Graham_Scan.h'
#include 'DandC_Convex_Hull.h'
#include 'Floyd_Eddy.h'
  type distributions = (rectangular,circular);
  var pointset,
      FE,
      pw,pw2,
      Gs,
      dac,work: SetOfPoints;
      hulls: Cursors;
      N,M,Mpackagewrapping,Mpackagewrapping2,MGrahamscan,Mdivideandconquer,
      TM,
      iterations,K,
      i,iseed:
	integer;
      start, FEtime, pwtime, pwtime2, Gstime, Gssorttime, dactime,
      TFEtime, Tpwtime, Tpwtime2, TGstime, TGssorttime, Tdactime,
      THpwtime, THpwtime2, THGstime, THGssorttime, THdactime:
        real;
      distribution: distributions;
      verbose: boolean;
      dummy: integer;
  begin
    writeln('Choose a distribution:');
    writeln(' <1> Uniform (over unit square)');
    writeln(' <2> Uniform (over unit circle)');
    write(' ---> '); readln(i);
    case i of
      1: distribution := rectangular;
      2: distribution := circular
    end;
    writeln('Distribution: ',distribution);
    write('Input seed for random number generator: ');
    readln(iseed);
    writeln('Seed is: ',iseed:1);
    dummy := seed(iseed);
    write('Input number of points: ');
    readln(N);
    writeln('Number of points is: ',N:1);
    write('Number of instances: ');
    readln(iterations);
    writeln('Number of instances is: ',iterations:1);
    write('Verbose (1) or summary (2) output: ');
    readln(K);
    verbose := K = 1;

    TM := 0;
    TFEtime := 0.0;
    Tpwtime := 0.0;
    Tpwtime2 := 0.0;
    TGstime := 0.0;
    TGssorttime := 0.0;
    Tdactime := 0.0;
    THpwtime := 0.0;
    THpwtime2 := 0.0;
    THGstime := 0.0;
    THGssorttime := 0.0;
    THdactime := 0.0;

    for K := 1 to iterations do
      begin
        for i := 1 to N do
          case distribution of
            rectangular:  begin
			    pointset[i].x := random(0.0);
	                    pointset[i].y := random(0.0)
		          end;
	    circular:     repeat
			    (* choose uniformly in containing square until
			       point falls into the circle *)
			    pointset[i].x := 2.0*random(0.0) - 1.0;
	                    pointset[i].y := 2.0*random(0.0) - 1.0
		          until sqr(pointset[i].x) + sqr(pointset[i].y) < 1
          end;
    
        for i := 1 to N do
          FE[i] := pointset[i];
        start := clock;
        FloydEddy(N,M,FE);
        FEtime := (clock - start)/1000.0;
	TM := TM + M;
	TFEtime := TFEtime + FEtime;

        for i := 1 to N do
          pw[i] := pointset[i];
        start := clock;
        PackageWrapping(N,Mpackagewrapping,pw);
        pwtime := (clock - start)/1000.0;
	Tpwtime := Tpwtime + pwtime;
(*
        for i := 1 to Mpackagewrapping do
          writeln('(',pw[i].x:1:3,',',pw[i].y:1:3,')');
        writeln;
*)

        for i := 1 to N do
          pw2[i] := pointset[i];
        start := clock;
        PackageWrapping2(N,Mpackagewrapping2,pw2);
        pwtime2 := (clock - start)/1000.0;
	Tpwtime2 := Tpwtime2 + pwtime2;
        if Mpackagewrapping <> Mpackagewrapping2
          then begin
	         writeln('ERROR package wrapping');
	         halt
               end;
        for i := 1 to Mpackagewrapping do
          if Compare(pw[i],pw2[i]) <> 0
	    then begin
	           writeln('ERROR package wrapping');
	           halt
	         end;
(*
        for i := 1 to Mpackagewrapping2 do
          writeln('(',pw2[i].x:1:3,',',pw2[i].y:1:3,')');
        writeln;
*)

        for i := 1 to N do
          Gs[i] := pointset[i];
        start := clock;
        GrahamScan(N,MGrahamscan,Gs,Gssorttime);
        Gstime := (clock - start)/1000.0;
	TGstime := TGstime + Gstime;
	TGssorttime := TGssorttime + Gssorttime;
        if Mpackagewrapping <> MGrahamscan
          then begin
	         writeln('ERROR Graham scan');
	         halt
               end;
        for i := 1 to Mpackagewrapping do
          if Compare(pw[i],Gs[i]) <> 0
	    then begin
	           writeln('ERROR Graham scan');
	           halt
	         end;
(*
        for i := 1 to MGrahamscan do
          writeln('(',Gs[i].x:1:3,',',Gs[i].y:1:3,')');
        writeln;
*)

        for i := 1 to N do
          dac[i] := pointset[i];
        start := clock;
        DivideAndConquer(N,Mdivideandconquer,dac,work,hulls);
        dactime := (clock - start)/1000.0;
	Tdactime := Tdactime + dactime;
        if Mpackagewrapping <> Mdivideandconquer
          then begin
	         writeln('ERROR divide and conquer');
	         halt
               end;
        for i := 1 to Mpackagewrapping do
          if Compare(pw[i],dac[i]) <> 0
	    then begin
	           writeln('ERROR divide and conquer');
	           halt
	         end;
(*
        for i := 1 to Mdivideandconquer do
          writeln('(',dac[i].x:1:3,',',dac[i].y:1:3,')');
        writeln;
*)

	if verbose
	  then begin
             writeln('Size of convex hull:                       ',
		      Mpackagewrapping:1);
             writeln('Package wrapping time:                     ',pwtime:1:2);
             writeln('Package wrapping time with heuristics:     ',pwtime2:1:2);
             writeln('Graham scan time:                          ',Gstime:1:2);
             writeln('Graham scan sort time:                     ',
		      Gssorttime:1:2);
             writeln('Divide and conquer time:                   ',dactime:1:2);
             writeln
	       end;

        for i := 1 to M do
          pw[i] := FE[i];
        start := clock;
        PackageWrapping(M,Mpackagewrapping,pw);
        pwtime := (clock - start)/1000.0;
	THpwtime := THpwtime + pwtime;
(*
        for i := 1 to Mpackagewrapping do
          writeln('(',pw[i].x:1:3,',',pw[i].y:1:3,')');
        writeln;
*)

        for i := 1 to M do
          pw2[i] := FE[i];
        start := clock;
        PackageWrapping2(M,Mpackagewrapping2,pw2);
        pwtime2 := (clock - start)/1000.0;
	THpwtime2 := THpwtime2 + pwtime2;
        if Mpackagewrapping <> Mpackagewrapping2
          then begin
	         writeln('ERROR package wrapping');
	         halt
               end;
        for i := 1 to Mpackagewrapping do
          if Compare(pw[i],pw2[i]) <> 0
	    then begin
	           writeln('ERROR package wrapping');
	           halt
                 end;
(*
        for i := 1 to Mpackagewrapping2 do
          writeln('(',pw2[i].x:1:3,',',pw2[i].y:1:3,')');
        writeln;
*)

        for i := 1 to M do
          Gs[i] := FE[i];
        start := clock;
        GrahamScan(M,MGrahamscan,Gs,Gssorttime);
        Gstime := (clock - start)/1000.0;
	THGstime := THGstime + Gstime;
	THGssorttime := THGssorttime + Gssorttime;
        if Mpackagewrapping <> MGrahamscan
          then begin
	         writeln('ERROR Graham scan');
	         halt
               end;
        for i := 1 to Mpackagewrapping do
          if Compare(pw[i],Gs[i]) <> 0
	    then begin
	           writeln('ERROR Graham scan');
	           halt
	         end;
(*
        for i := 1 to MGrahamscan do
          writeln('(',Gs[i].x:1:3,',',Gs[i].y:1:3,')');
        writeln;
*)

        for i := 1 to M do
          dac[i] := FE[i];
        start := clock;
        DivideAndConquer(M,Mdivideandconquer,dac,work,hulls);
        dactime := (clock - start)/1000.0;
	THdactime := THdactime + dactime;
        if Mpackagewrapping <> Mdivideandconquer
          then begin
	         writeln('ERROR divide and conquer');
	         halt
               end;
        for i := 1 to Mpackagewrapping do
          if Compare(pw[i],dac[i]) <> 0
	    then begin
	           writeln('ERROR divide and conquer');
	           halt
	         end;
(*
        for i := 1 to Mdivideandconquer do
          writeln('(',dac[i].x:1:3,',',dac[i].y:1:3,')');
        writeln;
*)

	if verbose
	  then begin
             writeln('After Floyd-Eddy number of points:         ',M:1);
             writeln('Floyd-Eddy heuristic time:                 ',FEtime:1:2);
             writeln('Size of convex hull:                       ',
		      Mpackagewrapping:1);
             writeln('Package wrapping time:                     ',pwtime:1:2);
             writeln('Package wrapping time with heuristics:     ',pwtime2:1:2);
             writeln('Graham scan time:                          ',Gstime:1:2);
             writeln('Graham scan sort time:                     ',
		      Gssorttime:1:2);
             writeln('Divide and conquer time:                   ',dactime:1:2);
	     writeln
	       end
      end;

writeln('           *** TOTALS WITHOUT FLOYD-EDDY***');
writeln('Package wrapping time:                     ',Tpwtime/iterations:1:2);
writeln('Package wrapping time with heuristics:     ',Tpwtime2/iterations:1:2);
writeln('Graham scan time:                          ',TGstime/iterations:1:2);
writeln('Graham scan sort time:                     ',
	 TGssorttime/iterations:1:2);
writeln('Divide and conquer time:                   ',Tdactime/iterations:1:2);
writeln;

writeln('           *** TOTALS WITH FLOYD-EDDY***');
writeln('After Floyd-Eddy number of points:         ',TM/iterations:1:2);
writeln('Floyd-Eddy heuristic time:                 ',TFEtime/iterations:1:2);
writeln('Package wrapping time:                     ',THpwtime/iterations:1:2);
writeln('Package wrapping time with heuristics:     ',THpwtime2/iterations:1:2);
writeln('Graham scan time:                          ',THGstime/iterations:1:2);
writeln('Graham scan sort time:                     ',
         THGssorttime/iterations:1:2);
writeln('Divide and conquer time:                   ',THdactime/iterations:1:2);
writeln
  end.
