#include 'Basic_Declarations.h'
#include 'Pseudo_Angle.h'
#include 'Convex_Hull_Decls.h'
#include 'DandC_Convex_Hull.h'

(* procedure DivideAndConquer(    N: integer; { number of points in input }
                              var M: integer; { number of points in hull }
                              var poly: SetOfPoints;

			      var work: SetOfPoints; { var for speed }
			      var start: Cursors); *)
procedure DivideAndConquer;
  (* Determine the convex hull of a polygon with a divide-and-conquer
     algorithm.  Rearranges points so that convex hull is first  M  points.

     As a matter of principle, routine can not know how big a work array to
     declare.  Solve this problem by having the user pass in  work.

     Assumes that all points are distinct and that there are at least
     two points (the two point case is degenerate).  Points may be collinear.

     Internal routines use the fact that the convex hull starts with the
     least (x,y) point and proceeds counterclockwise around the hull.  The
     answer is returned in this format as well.
  *)
  var i,j,k,
       excludedpoly,excludedwork, (* Starting locations in  poly  and  work
          where points interior to the hull are stored [from the right end]. *)
       lasthull: integer;
       inpoly: boolean;

  procedure MakeTriangle(var i: integer; (* starting location in  poly *)
                             k: integer); (* size of group in  poly *)
    (* At least two points and almost certainly three points. *)
    var min,j: integer;
         p: point;
    begin
      min := i;
      for j := i+1 to i+k-1 do
        if Compare(poly[j],poly[min]) < 0
          then min := j;
      p := poly[i]; poly[i] := poly[min]; poly[min] := p; (* swap *)
      if k = 2 (* rare case -- only two points originally *)
        then i := i + 2
        else begin
               if PolarCompare(poly[i],poly[i+1],poly[i+2]) > 0
                then begin
                       p := poly[i+1]; poly[i+1] := poly[i+2]; poly[i+2] := p
                     end;
               if Theta(poly[i],poly[i+1]) = Theta(poly[i],poly[i+2])
                 then begin (* three points are collinear -- hull has two
                               points *)
                        excludedpoly := excludedpoly - 1;
                        p := poly[i+1];
                        poly[i+1] := poly[i+2];
                        poly[i+2] := poly[excludedpoly];
                        poly[excludedpoly] := p;
                        i := i + 2
                      end
                 else i := i + 3
             end
    end;  (* MakeTriangle *)

  procedure merge(    start1,end1,start2,end2: integer;
                  var out,excluded: integer; var from,into: SetOfPoints);
    (* Merge two hulls into one using Graham scan, but taking advantage
       of presorted nature of the two hulls. *)
    label 99;
    var temp, (* for swapping *)
        i, (* scanning second hull for interior and potential hull points *)
        iplus1, (* i + 1 with wrap around *)
        numdone, (* number of break points found in second hull *)
        firstpoint,lastpoint, (* Indices of first and last points of second
                                 hull which might be in hull being formed.
                                 In counter-clockwise circular manner. *)
        j,k: integer;
        anchor: point; (* smallest (x,y) point on hull being formed *)
        l: segment;
        side1,side2: real;
    procedure Grahaminclude(j: integer; var i: integer);
      label 1;
      var l: segment;
      begin
        (* The new point is always included in the convex hull of points,
           since it has the highest angle or, in case of ties (collinear
           points with respect to the anchor), the point furthest from
           the anchor point.
        *)
        l.e2 := from[j];
        while i > out do
          begin
            l.e1 := into[i-1];
            if WhichSide(into[i],l) <= 0
              (* Points on the left are inside the convex hull.  If the point
                 is on the line then into[i-1], into[i] and from[j] are
                 collinear and into[i] can be eliminated.  The test  i > out
                 prevents subscript out of range if there are points with
                 the same  y  value as the anchor or the convex hull is
                 degenerate (a line segment).
              *)
              then begin
                     excluded := excluded - 1;
                     into[excluded] := into[i];
                     i := i - 1
                   end
              else goto 1 (* exit loop *)
          end;
     1: i := i + 1;
        into[i] := from[j]
      end; (* Grahaminclude *)

    begin
      (* Second hull is the one to be split and merged in. *)
      if Compare(from[start2],from[start1]) < 0
        then begin
               temp := start1;  start1  := start2;  start2  := temp;
               temp := end1;    end1 := end2;       end2 := temp
             end;
      (* Find the boundary between those points that can conceivably be
         on the hull and those that must be in the interior.  Use the
         smallest point of the first hull, which is the smallest point of
         either hull, to do this. *)
      (* There are two special cases that are a nuisance -- (1) points
         and (2) hulls consisting of two points, where the two points and the
         anchor are collinear. *)
      anchor := from[start1];
      if end2 = start2 (* length2 = 1 *)
        then begin
               firstpoint := start2;
               lastpoint  := start2
             end
      else if (end2 = start2 + 1) (* length2 = 2 *) and
          (Theta(anchor,from[start2]) = Theta(anchor,from[end2]))
        then begin
               firstpoint := start2 + 1;
               lastpoint  := start2 + 1;
               excluded := excluded - 1;
               into[excluded] := from[start2]
             end
      else begin (* default case -- taken almost all the time *)
             numdone := 0;
             i := start2;
             iplus1 := i + 1; (* second hull has at least two points *)
             l.e1 := from[i]; l.e2 := from[iplus1];
             side2 := WhichSide(anchor,l);
             while numdone < 2 do
               begin
                 side1 := side2;
                 i := iplus1;
     
                 (* find next point -- modular arithmetic *)
                 if iplus1 = end2
                   then iplus1 := start2
                   else iplus1 := iplus1 + 1;
                 l.e1 := l.e2; l.e2 := from[iplus1];

                 side2 := WhichSide(anchor,l);
                 if (* side1 <> side2 *)
		    (side1 * side2 < 0) or
		    ((side1 = 0) and (side2 <> 0)) or
		    ((side1 <> 0) and (side2 = 0))
                   then if side1 < 0
                          then begin
                                 lastpoint := i;
                                 numdone := numdone + 1
                               end
                        else if side2 < 0
                          then begin
                                 firstpoint := i;
                                 numdone := numdone + 1
                               end
                     (* else a case involving collinearity *)
               end;
             (* copy over the permanently excluded points -- points interior
                to the anchor and the widest expanse of the second convex
                hull.
             *)
             if lastpoint = end2
               then i := start2
               else i := lastpoint + 1;
             while i <> firstpoint do
               begin
                 excluded := excluded - 1;
                 into[excluded] := from[i];
                 if i = end2
                   then i := start2
                   else i := i + 1
               end
           end;
       
       (* Merge the first hull with what remains of the second hull using
          the Graham scan technique. *)
       i := out;
       into[i] := anchor;
       k := firstpoint;
       if start1 = end1 (* rare case *)
         then while true do
                begin
                  (* All of what remains of second hull in on new hull. *)
                  i := i + 1;
                  into[i] := from[k];
                  if k = lastpoint
                    then goto 99
                  else if k = end2
                    then k := start2
                  else   k := k + 1
               end;
       j := start1 + 1;
       while true do
         (* merge in the next point *)
         if PolarCompare(anchor,from[j],from[k]) < 0
           then begin
                  Grahaminclude(j,i);
                  if j = end1 (* hull 1 exhausted *)
                    then while true do
                           begin
                             Grahaminclude(k,i);
                             if k = lastpoint
                               then goto 99
                             else if k = end2
                               then k := start2
                             else   k := k + 1
                           end
                    else j := j + 1
                end
           else begin
                  Grahaminclude(k,i);
                  if k = lastpoint (* hull 2 exhausted *)
                    then while true do
                           begin
                             Grahaminclude(j,i);
                             if j = end1
                               then goto 99
                               else j := j + 1
                           end
                  else if k = end2
                    then k := start2
                  else   k := k + 1
                end;
  99: out := i + 1
    end; (* merge *)

  begin
    lasthull := 0;
    excludedpoly := N + 1;
    excludedwork := N + 1;

    (* Make trivial hulls out of next three (or less) elements. *)
    i := 1;
    (* while there are three or more points left *)
    while (* i + 2 <= excludedpoly - 1 *) i + 3 <= excludedpoly do
      begin
        lasthull := lasthull + 1;
        start[lasthull] := i;
        MakeTriangle(i,3)
      end;
    (* There are zero, one or two elements left. *)
    case excludedpoly - i of
      0: ;
      1: begin
           lasthull := lasthull + 1;
           start[lasthull] := i;
           i := i + 1
         end;
      2: begin
           lasthull := lasthull + 1;
           start[lasthull] := i;
           MakeTriangle(i,2)
         end
    end;
    start[lasthull+1] := i; (* where the next one would start if there were
                               a next one *)
    inpoly := true;

    while lasthull <> 1 do
      begin
        (* merge hulls two at a time *)
        j := 1;
        i := 1;
        while 2*j <= lasthull do
          begin
            start[j] := i; (* Does not overwrite, except for first hull,
                              which overwrites 1 with 1. *)
            if inpoly
              then merge(start[2*j-1],start[2*j] - 1,
                         start[2*j],start[2*j+1] - 1,
                         i,excludedwork,poly,work)
              else merge(start[2*j-1],start[2*j] - 1,
                         start[2*j],start[2*j+1] - 1,
                         i,excludedpoly,work,poly);
            j := j + 1
          end;
        (* if there is a left over hull copy it *)
        if 2*j - 1 = lasthull
          then begin
                 start[j] := i;
                 if inpoly
                   then for k := start[2*j-1] to start[2*j] - 1 do
                          begin
                            work[i] := poly[k];
                            i := i + 1
                          end
                   else for k := start[2*j-1] to start[2*j] - 1 do
                          begin
                            poly[i] := work[k];
                            i := i + 1
                          end;
                 j := j + 1
               end;
        start[j] := i;
        lasthull := j - 1;
        inpoly := not inpoly
      end;

    (* If result in wrong array copy it back.  Also transfer any interior
        points recorded in the work array *)
    for i := excludedwork to N do
      begin
        excludedpoly := excludedpoly - 1;
        poly[excludedpoly] := work[i]
      end;
    M := excludedpoly - 1;
    if not inpoly
      then for i := 1 to M do poly[i] := work[i]
  end; (* DivideAndConquer *)
