#include 'Basic_Declarations.h'
#include 'Convex_Hull_Decls.h'
#include 'Floyd_Eddy.h'

(* procedure FloydEddy(    N: integer; { number of points in input }
                       var M: integer; { number of points in output }
                       var S: SetOfPoints); *)
procedure FloydEddy;
  (* Use the Floyd-Eddy heuristic to reduce the size of a convex hull
     problem.
  *)
  label 1;
  var i,j,minx,maxx,miny,maxy,quadlen: integer;
      quad: array [1..4] of segment;
      p: point;

  begin
    (* Find the point of minimum x, maximum x, minimum y, and maximum y.
       For a problem of significant size these are almost certain to be
       distinct.  Except for problems of collinearity, these will all be
       on the convex hull.
    *)
    minx := 1; maxx := 1; miny := 1; maxy := 1;
    for i := 2 to N do
      begin
        if S[i].x < S[minx].x
          then minx := i
        else if S[i].x > S[maxx].x
          then maxx := i;
        if S[i].y < S[miny].y
          then miny := i
        else if S[i].y > S[maxy].y
          then maxy := i
      end;
    (* Ignore nonsensical cases that are legal.  All they do is make the
       heuristic (as applied) ineffective. *)
    quad[1].e1 := S[miny];
    quadlen := 1;
    if miny <> maxx
    then begin
           quadlen := quadlen + 1;
           quad[quadlen].e1 := S[maxx]
         end;
    if maxx <> maxy
    then begin
           quadlen := quadlen + 1;
           quad[quadlen].e1 := S[maxy]
         end;
    if maxy <> minx
    then begin
           quadlen := quadlen + 1;
           quad[quadlen].e1 := S[minx]
         end;
    if (quad[quadlen].e1.x = quad[1].e1.x) and
     (quad[quadlen].e1.y = quad[1].e1.y)
    then quadlen := quadlen - 1;
    for i := 1 to quadlen-1 do
      quad[i].e2 := quad[i+1].e1;
    quad[quadlen].e2 := quad[1].e1;

    i := 1;
    M := N;
    while i <= M do
      begin
        for j := 1 to quadlen do
          if WhichSide(S[i],quad[j]) >= 0
            then begin
                 (* point not in interior of quadralateral *)
                 i := i + 1;
                 goto 1
               end;
        (* This point in the interior and is permanently excluded *)
        p := S[i];
        S[i] := S[M];
        S[M] := p;
        M := M - 1;
   1: end
  end; (* FloydEddy *)
