#include 'Basic_Declarations.h'
#include 'Pseudo_Angle.h'
#include 'Convex_Hull_Decls.h'
#include 'Graham_Scan.h'

(* procedure GrahamScan(    N: integer; { Actual number of points in input }
                        var M: integer; { Number of points in hull }
                        var P: SetOfPoints;
			var time: real); *)
procedure GrahamScan;
  (* Assumes that all points are distinct and that there are at least
     two points (this last a degenerate case).  Points may be collinear.

     Points of P are rearranged so that the points that define the convex hull
     occupy the first M locations.
  *)
  label 1;
  type angles = array [1..MAX] of real;
  var i,j,k,m,min: integer;
      q: point;
      l: segment;
      A: angles; (* So that angles can be computed once and for all, which
		    makes the sorting more efficient, since then compares
		    are just comparisons.  FOR SPEED, ASSUME NO TIES ON THE
		    ANGULAR VALUES. *)
      start: integer;
      deleted: SetOfPoints; (* a lot of storage which will never be used *)
      d,d1: real;
  procedure quicksort(var A: angles; var S: SetOfPoints; N: integer);
    label 1, 2, 3, 4;
    const SMALLFILE = 16;
          (* Sizes between 6 and 25 show less than 10% difference
             in running times *)
          SMALLFILEminus1 = 15;
          STACKSIZE = 100;
    var T, temp: real;
	TP, tempP: point; 
        i, j, l, r, m: integer;
        i1,i2: integer;
        stack: array [1..STACKSIZE] of integer;
        (* Sufficient space to sort a list far in excess of the memory
           that can be placed on a computer *)
        top: integer;
    begin
      if N <= SMALLFILE
        then begin
               (* the smallest element (the anchor) is first, it acts as
		  a sentinel because the error condition for pseudo-angles
		  is -1. *)
	       for i1 := 3 to N do
	         (* A[2]/S[2] is also in (temporarily) correct place *)
	         begin
		   temp := A[i1]; tempP := S[i1];
		   i2 := i1-1;
		(* while PolarCompare(S[1],S[i2],tempP) > 0 do *)
		   while A[i2] > temp do
		     begin
		       A[i2+1] := A[i2]; S[i2+1] := S[i2];
		       i2 := i2 - 1
		     end;
		   A[i2+1] := temp; S[i2+1] := tempP
	         end;
	       goto 4
             end;
  
      (* Use quicksort to sort the file into small pieces that are internally
         jumbled, but with the pieces in correct relative order.
         Use insertion sort to sort these small pieces. *)
      top := 0;
      l := 2; (* anchor never moves *)
      r := N;
  
   1: (* l, m and r are all distinct *)
      (* Put the median element in A[l] so that we can start the hole at the
         left end.  Automatic sentinel in both directions of sweep. *)
      m := (l+r) div 2;
   (* if PolarCompare(S[1],S[m],S[l]) > 0 *)
      if A[m] > A[l] 
        then begin temp := A[l];  A[l] := A[m]; A[m] := temp;
                   tempP := S[l]; S[l] := S[m]; S[m] := tempP end;
   (* if PolarCompare(S[1],S[l],S[r]) > 0 *)
      if A[l] > A[r]
        then begin
               temp := A[l];  A[l] := A[r]; A[r] := temp;
               tempP := S[l]; S[l] := S[r]; S[r] := tempP;
               (* Only recompare S[m] and S[l] if S[r] was not biggest *)
            (* if PolarCompare(S[1],S[m],S[l]) > 0 *)
               if A[m] > A[l]
                 then begin temp := A[l];  A[l] := A[m]; A[m] := temp;
                            tempP := S[l]; S[l] := S[m]; S[m] := tempP end
             end;
      T := A[l]; (* Make a hole at S[l] *)
      TP := S[l];
  
      (* One of i and j points to the hole.  The other points to the last
         element of its portion of S[l..r].  Initially i points to the hole
         and j points to the last element of the portion that contains
         elements >= T *)
      i := l;
      j := r;
(* 2: repeat j := j - 1 until PolarCompare(S[1],S[j],TP) <= 0; *)
   2: repeat j := j - 1 until A[j] <= T;
      if j <= i
        then begin
               (* j = i-1  or possibly j = i *)
               A[i] := T; (* Fill in the hole *)
               S[i] := TP;
               i := i + 1; (* Element in A[i] is correctly placed *)
               goto 3
             end;
      A[i] := A[j]; (* Reestablish invariant by switching roles of i and j *)
      S[i] := S[j];
   (* repeat i := i + 1 until PolarCompare(A[1],A[i],T) >= 0; *)
      repeat i := i + 1 until A[i] >= T;
      if j <= i
        then begin
               (* i = j+1  or possibly i = j *)
               A[j] := T; (* Fill in the hole *)
               S[j] := TP;
               j := j - 1; (* Element in A[j] is correctly placed *)
               goto 3
             end;
      A[j] := A[i]; (* Reestablish invariant by switching roles of i and j *)
      S[j] := S[i];
      goto 2;
  
   3: (* Usually the element T is in neither part, but it is possible in rare
         circumstances that i-j = 1.  In this case no harm is done. *)
      (* Stack the larger and remove tail recursion to solve the smaller. *)
      if r - i > SMALLFILEminus1
        then if j - l > SMALLFILEminus1
               then begin
                      (* both subproblems large--stack larger subproblem *)
                      top := top + 2;
                      if j - l > r - i
                        then begin
                               stack[top-1] := l;
                               stack[top] := j;
                               l := i;
                               goto 1
                             end
                        else begin
                               stack[top-1] := i;
                               stack[top] := r;
                               r := j;
                               goto 1
                             end
                    end
               else begin
                      (* left subproblem is small--work on right subproblem *)
		      (* either there is a subpiece to the left or the anchor
		         is a sentinel *)
	              for i1 := l+1 to j do
	                begin
		          temp := A[i1]; tempP := S[i1];
		          i2 := i1-1;
		       (* while PolarCompare(S[1],S[i2],tempP) > 0 do *)
		          while A[i2] > temp do
		            begin
		              A[i2+1] := A[i2];
		              S[i2+1] := S[i2];
		              i2 := i2 - 1
		            end;
		          A[i2+1] := temp;
		          S[i2+1] := tempP
	                end;
                      l := i;
                      goto 1
                    end
        else begin
               (* right subproblem is small *)
	       (* there is a subpiece to the left to act as a sentinel *)
	       for i1 := i+1 to r do
	         begin
		   temp := A[i1]; tempP := S[i1];
		   i2 := i1-1;
		(* while PolarCompare(S[1],S[i2],tempP) > 0 do *)
		   while A[i2] > temp do
		     begin
		       A[i2+1] := A[i2];
		       S[i2+1] := S[i2];
		       i2 := i2 - 1
		     end;
		   A[i2+1] := temp;
		   S[i2+1] := tempP
	         end;
               if j - l > SMALLFILEminus1
                 then begin (* left subproblem large -- go work on it *)
                        r := j;
                        goto 1
                      end
                 else begin
                        (* left subproblem also small *)
		        (* either there is a subpiece to the left or the anchor
		           is a sentinel *)
	                for i1 := l+1 to j do
	                  begin
		            temp := A[i1]; tempP := S[i1];
		            i2 := i1-1;
		         (* while PolarCompare(A[1],A[i2],temp) > 0 do *)
		            while A[i2] > temp do
		              begin
		                A[i2+1] := A[i2];
		                S[i2+1] := S[i2];
		                i2 := i2 - 1
		              end;
		            A[i2+1] := temp;
		            S[i2+1] := tempP
	                  end;
                        (* retrieve an unsolved problem from the stack *)
                        if top > 0
                          then begin
                                 l := stack[top-1];
                                 r := stack[top];
                                 top := top - 2;
                                 goto 1
                               end
                      end
             end;
  
      (* entire file is sorted *)
  4: (* exit for files sorted entirely by insertion sort *)
    end; (* quicksort *)
  begin
    (* Find the minimum point -- smallest y and, within equal y values,
       smallest x.  Swap this point with the first point. *)
    min := 1;
    for i := 2 to N do
      if Compare(P[i],P[min]) < 0 then min := i;
    q := P[1]; P[1] := P[min]; P[min] := q;

    (* Compute angles once and for all and then sort *)
    for i := 1 to N do A[i] := Theta(P[1],P[i]);
    start := clock;
    quicksort(A,P,N);
    time := (clock - start)/1000.0;

    (* Deal with ties amongst the angles (there won't be any in random
       data). *)
    j := 0;
    m := 0;
    i := 1;
    while i <= N-1 do
      if A[i] = A[i+1]
        then begin
	       (* Find the point furthest from the anchor *)
	       k := i;
	       d := abs(P[1].x - P[k].x) + abs(P[1].y - P[k].y);
	       i := i + 1;
	       while A[i] = A[k] do
		 begin
		   j := j + 1; (* will eliminate one of the two points *)
		   d1 := abs(P[1].x - P[i].x) + abs(P[1].y - P[i].y);
		   if d < d1
		     then begin
			    deleted[j] := P[k];
			    d := d1;
			    k := i
			  end
		     else deleted[j] := P[i];
		   i := i + 1
		 end;
	       m := m + 1;
	       P[m] := P[k]
	     end
        else begin
	       m := m + 1;
	       P[m] := P[i];
	       i := i + 1
	     end;
    if i = N (* last element was not involved in a tie *)
      then begin
	     m := m + 1;
	     P[m] := P[i]
	   end;
    (* put deleted points at the end *)
    k := m;
    for i := 1 to j do
      begin
	k := k + 1;
	P[k] := deleted[i]
      end;

    (* The stack occupies the M positions of P *);
    M := 2; (* Initialize the stack to P[1] and P[2] *)
    for i := 3 to m do
      begin
        (* The new point is always included in the convex hull of points
           P[1..i] since it has the highest angle and, in case of ties
           (collinear points with respect to the anchor), it is also
           the point furthest from the anchor point. *)
        l.e2 := P[i];
	(* Eliminate points from the hull (stack) due to inclusion of P[i] *)
        while M > 1 do
          begin
            l.e1 := P[M-1];
            if WhichSide(P[M],l) <= 0
              (* Points on the left are inside the convex hull.
                 If the point is on the line then P[M-1], then
                 P[M] and P[i] are collinear and P[M] is eliminated.
                 The test  M > 1 prevents subscript problems
                 if there are collinear points with minimum angle. *)
              then M := M - 1
              else goto 1 (* exit loop *)
          end;
     1: (* Place P[i] on the stack *)
	M := M + 1;
        q := P[M]; P[M] := P[i]; P[i] := q
      end
  end; (* GrahamScan *)
