#include 'Basic_Declarations.h'
#include 'Pseudo_Angle.h'
#include 'Convex_Hull_Decls.h'
#include 'Package_Wrapping.h'

(* procedure PackageWrapping(    N: integer; { number of points in input }
                             var M: integer; { number of points in hull }
                             var P: SetOfPoints); *)
procedure PackageWrapping;
  (* Determine the convex hull of a polygon with the package wrapping
     algorithm.  Rearranges points so that convex hull is first  M  points.

     Assumes that all points are distinct and that there are at least
     two points (the two point case is degenerate).  Points may be collinear.

     Procedure given in Sedgewick.
  *)
  var i,min: integer;
      currentangle,minangle,t: real; (* pseudo-angles *)
      p: point;
  begin
    min := 1; (* Find the minimum point -- smallest  y  and within equal  y
                 values smallest  x. *)
    for i := 2 to N do
      if Compare(P[i],P[min]) < 0
        then min := i;
    p := P[1]; P[1] := P[min]; P[min] := p;
    
    M := 1;
    minangle := -1.0; (* So that all angles are considered on first pass. *)
    repeat
      (* Desire point with smallest angle greater than current angle. *)
      currentangle := minangle;
      if M > 1
	then minangle := Theta(P[M],P[1]) (* Try to return home *)
	else minangle := maxint;
      min := 1;
      for i := M+1 to N do
        begin
          t := Theta(P[M],P[i]);
          if t > currentangle
            then if t < minangle
                   then begin
                          minangle := t;
                          min := i
                        end
                 else if t = minangle
                   (* Deal with collinearity -- go with farthest point along
                      this ray.  Use Manhattan metric instead of Euclidean
                      metric, which is okay due to theorems on similar
                      triangles. *)
                   then if abs(P[i].x - P[M].x) + abs(P[i].y - P[M].y) >
                           abs(P[min].x - P[M].x) + abs(P[min].y - P[M].y)
                          then min := i
        end;
      M := M + 1;
      p := P[min]; P[min] := P[M]; P[M] := p
    until min = 1; (* until we have closed the polygon *)
    (* Undo the last swap, since we need to return all the original points
       to the caller. *)
    p := P[min]; P[min] := P[M]; P[M] := p;
    M := M - 1
  end; (* PackageWrapping *)
