#include 'Basic_Declarations.h'
#include 'Pseudo_Angle.h'
#include 'Convex_Hull_Decls.h'
#include 'Package_Wrapping2.h'

(* procedure PackageWrapping2(    N: integer; { number of points in input }
                              var M: integer; { number of points in hull }
                              var P: SetOfPoints); *)

procedure PackageWrapping2;
  (* Determine the convex hull of a polygon with the package wrapping
     algorithm.  Rearranges points so that convex hull is first  M  points.

     Assumes that all points are distinct and that there are at least
     two points (the two point case is degenerate).  Points may be collinear.

     Procedure given in Sedgewick with additional test to eliminate points
     in the interior as soon as that fact can be ascertained.
  *)
  label 99;
  var i,min,currentN: integer;
      currentangle,minangle,backangle,t: real; (* pseudo-angles *)
      p: point;
  begin
    min := 1; (* Find the minimum point -- smallest  y  and within equal  y
                 values smallest  x. *)
    for i := 2 to N do
      if Compare(P[i],P[min]) < 0
        then min := i;
    p := P[1]; P[1] := P[min]; P[min] := p;

    currentN := N; (* points beyond  currentN  are known to be interior *)
    M := 1;
    minangle := -1.0; (* so that all angles are considered on first pass *)
    while true do
      begin
        (* Desire point with smallest angle greater than current angle. *)
        currentangle := minangle;
	if M <> 1
	  then backangle := Theta(P[M],P[1])
	  else backangle := maxint;
        min := 1;
        minangle := backangle;
        i := M + 1;
        while i <= currentN do
          begin
            t := Theta(P[M],P[i]);
            if t > currentangle
              then if t < minangle
                     then begin
                            minangle := t;
                            min := i;
                            i := i + 1
                          end
                   else if t = minangle
                     (* Deal with collinearity -- go with farthest point along
                        this ray.  Use Manhattan metric instead of Euclidean
                        metric, which is okay due to theorems on similar
                        triangles. *)
                     then (* one of two points not on hull *)
                          if abs(P[i].x - P[M].x) + abs(P[i].y - P[M].y) >
                             abs(P[min].x - P[M].x) + abs(P[min].y - P[M].y)
                            then begin
                                   p := P[min];
                                   P[min] := P[i];
                                   P[i] := P[currentN];
                                   P[currentN] := p;
                                   currentN := currentN - 1
                                 end
                            else begin
                                   p := P[i];
                                   P[i] := P[currentN];
                                   P[currentN] := p;
                                   currentN := currentN - 1
                                 end
                   else if t >= backangle
                          then begin
                                 (* point in the interior if we close the
                                    polygon at this time *)
                                 p := P[i];
                                 P[i] := P[currentN];
                                 P[currentN] := p;
                                 currentN := currentN - 1
                               end
                   else (* point remains in contention for the future *)
                        i := i + 1
              else (* point in the interior if we close the polygon at
                      this time *)
                   begin
                     p := P[i];
                     P[i] := P[currentN];
                     P[currentN] := p;
                     currentN := currentN - 1
                   end
          end;
	if min <> 1
	  then begin
                 M := M + 1;
                 p := P[min]; P[min] := P[M]; P[M] := p
	       end
	  else goto 99
      end;
99:
  end; (* PackageWrapping2 *)
