#include 'Basic_Declarations.h'
#include 'Pseudo_Angle.h'

(* function Theta(p1,p2: point): real; *)
function Theta;
  (* Return the pseudo-angle between the y-axis, with orientation down as
     positive, p1 (as origin) and p2.

     The pseudo-angle has the property that the ordering of points by
     true angle around the origin and the ordering of points by the
     pseudo-angle are the same.

     Result returned is in [0,4) (or error condition).
  *)

  var dx,dy,t: real;
  begin
    dx := p2.x - p1.x;
    dy := p2.y - p1.y;
    if (dx = 0.0) and (dy = 0.0)
      then Theta := -1.0 (* error condition -- degenerate *)
      else begin
             t := dx/(abs(dx) + abs(dy));
             (* correct for quadrant -- fourth quadrant [0,1] *)
             if dy > 0.0 (* first quadrant: (1,2]
                            second quadrant:  [2,3) *)
               then t := 2.0 - t
             else if dx < 0.0 (* third quadrant: [3,4) *)
               then t := 4.0 + t;
             Theta := t
           end
  end; (* Theta *)

(* function PolarCompare(p,p1,p2: point): real; *)
function PolarCompare;
  (* Using  p  as the origin determine the relationship between  p1  and
      p2  in polar coordinates.  Primary key is angle (with downwards
      y-axis being the logical x-axis) and secondary key is radius.
  *)
  var comp: real;
  begin
    comp := Theta(p,p1) - Theta(p,p2);
    if comp = 0
      then (* By theorem on similar triangles can use Manhattan metric *)
	   comp := ( abs(p1.x - p.x) + abs(p1.y - p.y) ) -
	           ( abs(p2.x - p.x) + abs(p2.y - p.y) );
    PolarCompare := comp
  end; (* PolarCompare *)
