procedure GrahamScan(    n: integer; (* actual number of points in input *)
                     var m: integer; (* number of points in hull *)
                     var P: SetOfPoints);
  (* Points may be collinear and need not even be distinct.  The points of P
     are rearranged so that the m convex hull points come first. *)
  label 1;
  var i, min: integer;
      q: point;
      l: segment;
  begin
    (* Find the minimum point--smallest y and, within equal y values,
       smallest x.  Swap this point with the first point. *)
    min := 1;
    for i := 2 to n do if Compare(P[i],P[min]) < 0 then min := i;
    q := P[1]; P[1] := P[min]; P[min] := q;

    (* Sort the points using polar coordinates, treating the anchor as the
       origin.  The primary key is the pseudoangle of the line from the anchor
       to the point; the secondary key is the distance from the anchor. *)
    ... not shown ...

    (* The stack contains points thought to be on the hull and occupies
       the first m positions of P; initially, it contains P1 and P2. *);
    m := 2; (* Effectively push P[1] and P[2] onto the stack. *)
    for i := 3 to n do
      begin
        (* The new point is always included in the convex hull of the points
           in P[1..i], because it has the largest angle and, in case of a tie,
           is farthest from the anchor. *)
        l.e2 := P[i];
        (* Eliminate points from the hull (stack) due to the inclusion of P[i]. *)
        while m > 1 do
          begin
            l.e1 := P[m-1];
            if WhichSide(P[m],l) <= 0
              (* Points on the left are inside the convex hull.  As we have m > 1,
                 there cannot be any subscript problems due to collinearity. *)
              then m := m - 1
              else goto 1
          end;
     1: (* Place P[i] on the stack. *)
        m := m + 1;
        q := P[m]; P[m] := P[i]; P[i] := q
      end
  end; (* GrahamScan *)
