function Theta(p1, p2: point): real;
  (* Return the pseudoangle between the line from p1 to (infinity,p1.y)
     and the line from p1 to p2.  The pseudoangle has the property that
     the ordering of points by true angle around p1 and the ordering of
     points by pseudoangle are the same. *)
  (* The result is in the range [0,4) (or it is an error indication). *)
  var dx, dy, t: real;
  begin
    dx := p2.x - p1.x;
    dy := p2.y - p1.y;
    if (dx = 0.0) and (dy = 0.0)
      then Theta := -1.0 (* error indication *)
      else begin
             t := dy/(abs(dx) + abs(dy));
             (* Now correct for quadrant--first quadrant: [0,1] *)
             if dx < 0.0 (* inside second or third quadrant: (1,3) *)
               then t := 2.0 - t
             else if dy < 0.0 (* fourth quadrant: [3,4) *)
               then t := 4.0 + t;
             Theta := t
           end
  end; (* Theta *)
