const MaxR = ...; (* maximum number of rectangles *)
      MaxSkylineSize = ...; (* 4*MaxR-1 *)
type skyline = array [1..MaxSkylineSize] of real;

procedure Merge(var S1, S2: skyline;
                    n1, n2: integer;
                var S: skyline;
                var n: integer);
  (* Merge skylines S1[1..n1] and S2[1..n2] into S[1..n].
     Data are in the form x1, y1, x2, y2, ..., xk,
     so n1 and n2 are necessarily odd and >= 3. *)
  var i, (* points to first unprocessed abscissa of skyline 1 *)
      j, (* points to first unprocessed abscissa of skyline 2 *)
      m: integer;
      k: 1..2; (* Which skyline is currently contributing to the output. *)
      CurHeight1, CurHeight2, CurHeight: real;
  begin
    i := 1; j := 1; n := 0;
    CurHeight1 := 0; CurHeight2 := 0; CurHeight := 0;
    k := 1; (* arbitrarily start with skyline 1 *)
    while (i <= n1) and (j <= n2) do
      (* case S1[i] vs. S2[j] of *)
      if S1[i] < S2[j]
        then begin
               if i < n1 then CurHeight1 := S1[i+1] else CurHeight1 := 0;
                 (* Establish sentinel at end of skyline. *)
               if k = 1
                 (* Skyline can: move up and stay with skyline 1,
                                 move down and stay with skyline 1,
                                 move down and switch to skyline 2, or
                                 stay level and switch to skyline 2. *)
                 then if CurHeight1 >= CurHeight2
                        then begin (* Stay with skyline 1; move up or down. *)
                               S[n+1] := S1[i]; S[n+2] := CurHeight1;
                               CurHeight := CurHeight1;
                               n := n + 2
                             end
                        else begin (* Switch to skyline 2. *)
                               k := 2;
                               if CurHeight2 < CurHeight
                                 then begin (* skyline moves down *)
                                        S[n+1] := S1[i]; S[n+2] := CurHeight2;
                                        CurHeight := CurHeight2;
                                        n := n + 2
                                      end
                              (* else x value is dropped from the output, because
                                      the skyline accidentally stays level. *)
                             end
                 else (* k = 2 *)
                      (* Skyline can: move up and switch to skyline 1 or
                                      stay level. *)
                      if CurHeight1 > CurHeight
                        then begin
                               S[n+1] := S1[i]; S[n+2] := CurHeight1;
                               CurHeight := CurHeight1;
                               k := 1;
                               n := n + 2
                             end;
                     (* else x value is dropped from the output, because
                             skyline 2 continues to dominate;
                             this is the source of output-sensitivity. *)
               i := i + 2
             end
      else if S1[i] > S2[j]
        then ... not shown ... (* symmetric situation *)
      else   begin (* S1[i] = S2[j] *)
               (* Skyline can: move up,
                               move down, or
                               stay level, but switch skylines *)
               if i < n1 then CurHeight1 := S1[i+1] else CurHeight1 := 0;
               if j < n2 then CurHeight2 := S2[j+1] else CurHeight2 := 0;
               if CurHeight1 >= CurHeight2
                 then begin
                        k := 1;
                        if CurHeight1 <> CurHeight (* up or down *)
                          then begin
                                 S[n+1] := S1[i];
                                 if (i<>n1) or (j<>n2) then S[n+2] := CurHeight1;
                                 CurHeight := CurHeight1;
                                 n := n + 2
                               end
                      end
                 else begin
                        k := 2;
                        if CurHeight2 <> CurHeight (* up or down *)
                          then begin
                                 S[n+1] := S2[j]; S[n+2] := CurHeight2;
                                 CurHeight := CurHeight2;
                                 n := n + 2
                               end
                      end;
               i := i + 2; j := j + 2
             end;
    (* Copy remaining portion of either S1 or S2. *)
    for m := i to n1 do begin S[n] := S1[m]; n := n + 1 end;
    for m := j to n2 do begin S[n] := S2[m]; n := n + 1 end
  end; (* Merge *)
